;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Jezu Ufam Tobie
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; program : JUT2 -> 3d rotozoomer. Textured parallelpiped
;;           with bilinear filtering, illuminated by three
;;           unlinear lights + lines with interpolated
;;           color and transparency (rays).
;;           Thanks to Pablo Reda for simple line procedure.
;; author  : Maciej Guba aka macgub
;; web     : www.macgub.vxm.pl
;; email   : macgub3@wp.pl
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Credits:
;;   Big thanks to Jan Pawel II, Tomasz Grysztar, Mikolaj
;;   Feliks, Pablo Reda, Ica, Chem, lostcauz, tthsqe and
;;   all from board.flatassembler.net.
;;   It's 32 bit sse3 app.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

format PE GUI 4.0
entry start

include '/include/win32a.inc'

TEX_X = 256    ; texture size
TEX_Y = 425
RAYS_COUNT = 1600   ; divisable by 4
TEXTURE_SIZE EQU (TEX_X * TEX_Y)-1
TEX_SHIFT = 8
ROUND equ 8
WM_FINISH equ WM_USER+100h

section '.text' code readable executable
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; start and message loop
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
start:

    invoke GetModuleHandle,0
    mov [wc.hInstance],eax
    invoke LoadIcon,0,IDI_APPLICATION
    mov [wc.hIcon],eax
    invoke LoadCursor,0,IDC_ARROW
    mov [wc.hCursor],eax

    rdtsc
    mov     [rand_seed],ax  ; randomize

    invoke  GetSystemMetrics,SM_CXSCREEN
    sub     eax,10
    and     eax,0xfffffffe

    push    eax
    mov     [xres_var],ax
    shr     eax,1
    mov     [xxadd],ax
    invoke  GetSystemMetrics,SM_CYSCREEN
    sub     eax,10
    and     eax,0xfffffffe

    push    eax
    mov     [yres_var],ax
    mov     ecx,eax
    shr     ecx,1
    mov     [yyadd],cx

    movzx  eax,word[xres_var]
    movzx  ebx,[yres_var]
    imul   eax,ebx
    shl    eax,2
    mov    [screen_size],eax

    pop eax
    pop ebx

    mov [bmi.biSize],sizeof.BITMAPINFOHEADER
    mov [bmi.biWidth],ebx
    neg eax
    mov [bmi.biHeight],eax
    mov [bmi.biPlanes],1
    mov [bmi.biBitCount],32
    mov [bmi.biCompression],BI_RGB

    mov [bmi.biSizeImage],0
    mov [bmi.biXPelsPerMeter],0
    mov [bmi.biYPelsPerMeter],0
    mov [bmi.biClrUsed],0
    mov [bmi.biClrImportant],0

    mov [rect.left],0
    mov [rect.top],0
    mov [rect.right],ebx
    neg eax
    mov [rect.bottom],eax
    invoke AdjustWindowRect,rect,WS_POPUP or WS_SYSMENU or WS_CAPTION,0 

    invoke RegisterClass,wc

    mov ecx,[rect.bottom]
    mov eax,[rect.right]
    sub ecx,[rect.top]
    sub eax,[rect.left] 

    invoke CreateWindowEx,0,_class,_title,WS_OVERLAPPEDWINDOW\
           xor WS_THICKFRAME xor WS_MAXIMIZEBOX,0,0,eax,ecx,NULL,NULL,0,0
    mov [hwnd],eax
    invoke GetDC,[hwnd]
    mov [hdc],eax
    invoke ShowWindow,[hwnd],SW_SHOW


     movzx   eax,word[xres_var]
     movzx   ebx,[yres_var]
     mul     ebx
     shl     eax,2
     mov     ebx,eax
     invoke  VirtualAlloc,NULL,eax,MEM_COMMIT+MEM_RESERVE,PAGE_EXECUTE_READWRITE
     or     eax,eax
     jnz    @f
     int3
   @@:
     add     eax,16
     and     eax,0xfffffff0
     mov     [screen_ptr],eax


     invoke  VirtualAlloc,NULL,ebx,MEM_COMMIT+MEM_RESERVE,PAGE_EXECUTE_READWRITE
     or  eax,eax
     jnz @f
     int3
   @@:
     add     eax,16
     and     eax,-16
     mov     [Zbuffer_ptr],eax

    call random_rays

     push ebp
     push dword points_r

     movzx    eax,[yres_var]
     shr      eax,2
     cvtsi2ss xmm0,eax
     movss  [scale],xmm0

    mov esi,jut
    mov edi,texmap
    mov ecx,TEX_Y*TEX_X
  @@:
    mov eax,[esi]
    mov [edi],eax
    add edi,4
    add esi,3
    sub ecx,1
    jnz @b


   mov  edi,lights_aligned
   mov  ecx,LIGHTS_COUNT
   call norm_aligned_lv


    pushad
    push   dword points_r
    movzx  ecx,[points_count_var]
    add    cx,[rays_count_var]
    push   ecx
    call   normalize_object1
    popad


    call   init_triangles_normals2
    call   init_point_normals

msg_loop:

    stdcall g_draw
if 0 ; debug
    invoke CreateThread,NULL,NULL,gdraw_th,\
           NULL,NORMAL_PRIORITY_CLASS,\
           [ThreadID_gdraw]
    mov    [hthread_gdraw],eax
    invoke SetThreadPriority,hthread_gdraw,THREAD_PRIORITY_TIME_CRITICAL
    invoke SetThreadAffinityMask,[hthread_gdraw],0
    invoke ResumeThread,[hthread_gdraw]
    invoke WaitForMultipleObjectsEx,1,hthread_gdraw,1,-1,0
    invoke CloseHandle,hthread_gdraw   ;2

end if

    add  [ccounter],1
    cmp  [ccounter],0x1ff

    movzx  eax,word[xres_var]
    movzx  ebx,[yres_var]
    invoke StretchDIBits,[hdc],0,0,eax,ebx,0,0,eax,ebx,[screen_ptr],bmi,0,SRCCOPY
                                           ; debug -   [screen_ptr]
    cmp    [speed_flag],1
    je     @f
    invoke Sleep,3
  @@:

    invoke PeekMessage,msg,0,0,0,PM_NOREMOVE
    or eax,eax
    je msg_loop

    invoke GetMessage,msg,NULL,0,0
    or eax,eax
    je end_loop

    invoke TranslateMessage,msg
    invoke DispatchMessage,msg
    jmp msg_loop

error:
    invoke MessageBox,NULL,_error,NULL,MB_ICONERROR+MB_OK

end_loop:
    invoke ReleaseDC,[hwnd],[hdc]
    invoke DestroyWindow,[hwnd]
    invoke ExitProcess,[msg.wParam]
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; WindowProc
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
proc WindowProc uses ebx esi edi, hwnd,wmsg,wparam,lparam
    cmp [wmsg],WM_KEYDOWN
    je .wmkeydown
    cmp [wmsg],WM_DESTROY
    je .wmdestroy
  .defwndproc:
    invoke DefWindowProc,[hwnd],[wmsg],[wparam],[lparam]
    jmp .finish
  .wmkeydown:
    cmp [wparam],VK_UP
    jne .vescape

    xor eax,eax
    jmp .finish
  .vescape:

    cmp [wparam],VK_SPACE
    jne @f
    add [speed_flag],1
    and [speed_flag],1
   @@:


    cmp [wparam],VK_ESCAPE
    jne .finish
  .wmdestroy:
    invoke PostQuitMessage,0
    xor eax,eax
  .finish:
    ret
endp

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; g_draw - draw to screen buffer; main program proc
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
proc g_draw

    push ebp
    mov  ebp,esp
    sub  esp,64
    and  ebp,0xfffffff0


    pushad

    xor   eax,eax          ; clear draw buffer
    mov   edi,[screen_ptr]
    movzx ebx,[yres_var]
    movzx ecx,word[xres_var]
    imul  ecx,ebx
    call  fill_aligned

    mov   eax,60000.0
    mov   edi,[Zbuffer_ptr]
    movzx ebx,word[yres_var]
    movzx ecx,word[xres_var]
    imul  ecx,ebx
    call  fill_aligned

    fninit
    fild   [ccounter]
    fmul   [one_deg]

    fld    st
    fsincos
    fst   dword[cos_beta]
    fchs
    fstp  [neg_cos_beta]
    fst   [sin_beta]
    fchs
    fstp  [neg_sin_beta]

  @@:
    fld1
    fld1
    faddp
    fmulp

    fsincos
    fst   dword[cos_gamma]
    fchs
    fstp  [neg_cos_gamma]
    fst   [sin_gamma]
    fchs
    fstp  [neg_sin_gamma]

 ; .rot_points
    mov    esi,points_r         ; rotate points
    mov    edi,points_rotated
    movzx  ecx,[points_count_var]
    call   rotor
 ;.rot_normals:
    mov    esi,points_normals
    mov    edi,points_normals_rotated
    movzx  ecx,[points_count_var]
    call   rotor


    mov    esi,rays_vertices
    mov    edi,rays_vertices_rotated
    movzx  ecx,[rays_count_var]
    add    ecx,2
    call   rotor



    cmp   [scale_cnt],912       ; scale counter
    jne   @f
    mov   [scale_dir],1         ; scale direction
    jmp   .sc1
  @@:
    cmp   [scale_cnt],0
    jne   @f
    mov   [scale_dir],0
  @@:
  .sc1:
    movss xmm0,[scale]
    cmp   [scale_dir],0
    jne   @f
    mulss xmm0,[scale_inc]
    add   [scale_cnt],1
    jmp   .finish_scale
  @@:
    mulss xmm0,[scale_dec]
    sub   [scale_cnt],1
  .finish_scale:
    movss  [scale],xmm0


   ;translate
      mov          esi,points_rotated
      movzx        ecx,[points_count_var]
      movq         xmm0,[xxadd]
      movss        xmm3,[scale]
      call         translate_and_scale_vert

      mov          esi,rays_vertices_rotated
      movzx        ecx,[rays_count_var]
      add          ecx,2
      movq         xmm0,[xxadd]
      movss        xmm3,[scale]
      call         translate_and_scale_vert


 ; stencil buffer -> z buffer
    mov    esi,triangles
    movzx  ecx,[triangles_count_var]



;   mov  eax,1
;   call draw_thread



if 1 ; debug
    invoke CreateThread,NULL,NULL,draw_th0,\
           NULL,NORMAL_PRIORITY_CLASS,\
           [ThreadID0]
    mov    [hthread0],eax
    invoke  SetThreadPriority,hthread0,THREAD_PRIORITY_TIME_CRITICAL

   invoke CreateThread,NULL,NULL,draw_th1,\
           NULL,NORMAL_PRIORITY_CLASS,\
           [ThreadID1]
    mov  [hthread1],eax
    invoke  SetThreadPriority,hthread1,THREAD_PRIORITY_TIME_CRITICAL

    invoke SetThreadAffinityMask,[hthread0],1
    invoke SetThreadAffinityMask,[hthread1],2
    invoke ResumeThread,[hthread0]
    invoke ResumeThread,[hthread1]
    invoke WaitForMultipleObjectsEx,2,hthread0,1,-1,0
    invoke CloseHandle,hthread0   ;2
    invoke CloseHandle,hthread1
end if
    emms
    mov     edi,rays
    mov     ecx,0
  .again_ray:
    push    edi
    mov     esi,ecx
    push    ecx
    movzx   eax,word[edi]
    movzx   ebx,word[edi+2]
    imul    eax,[i12]
    imul    ebx,[i12]
    add     eax,rays_vertices_rotated
    add     ebx,rays_vertices_rotated
    movd    mm4,[screen_size]
    movups  xmm0,[eax]
    movups  xmm1,[ebx]
    movhlps xmm3,xmm0
    movhlps xmm2,xmm1
    cvtps2dq  xmm0,xmm0
    cvtps2dq  xmm1,xmm1
    movd      eax,xmm0
    movd      ecx,xmm1
    psrldq    xmm0,4
    movd      ebx,xmm0
    psrldq    xmm1,4
    movd      edx,xmm1
    push      eax
    movzx     edi,word[xres_var]
    movd      mm2,edi
    movd      mm0,[Zbuffer_ptr]
    movd      mm1,[screen_ptr]

    movd      xmm7,esi
    pshufd    xmm7,xmm7,0
    movdqa    xmm6,xmm7
    pcmpgtd   xmm6,[ray_clip1]  ;mm7
    pcmpgtd   xmm7,[ray_clip2]  ;mm6
    pxor      xmm7,xmm6
    movmskps  eax,xmm7
    and       eax,11b
    or        eax,eax
    jz        @f
    movaps    xmm0,[color1]
    jmp       .red
  @@:
    movaps    xmm0,[color3]
  .red:
    movaps    xmm1,[color2]
    pop    eax
    call   line_grd_trans
    pop    ecx
    pop    edi
    add    edi,4
    add    ecx,1
    cmp    cx,RAYS_COUNT ; shr 1 ;[rays_count_var]
    jnz    .again_ray


if 0
;====================write trianles normals
    movzx   ecx,[triangles_count_var]
    mov     ebx,triangles
    mov     edi,triangles_normals_rotated
    emms
  @@:
    push    ecx
    push    edi
    push    ebx
    movzx   edx,word[ebx]
    imul    edx,[i12]
    movups  xmm0,[edx+points_rotated]
    movzx   edx,word[ebx+2]
    imul    edx,[i12]
    movups  xmm1,[edx+points_rotated]
    movzx   edx,word[ebx+4]
    imul    edx,[i12]
    movups  xmm2,[edx+points_rotated]
    mov     edx,3
    cvtsi2ss  xmm3,edx
    addps   xmm0,xmm1
    addps   xmm0,xmm2
    shufps  xmm3,xmm3,0
    divps   xmm0,xmm3


    movups  xmm1,[edi]
    mov     eax,30.0
    movd    xmm3,eax
    pshufd  xmm3,xmm3,0
    mulps   xmm1,xmm3
    addps   xmm1,xmm0
    cvtps2dq xmm0,xmm0
    cvtps2dq xmm1,xmm1
    movd    eax,xmm0
    movd    ecx,xmm1
    psrldq  xmm0,4
    psrldq  xmm1,4
    movd    ebx,xmm0
    movd    edx,xmm1
    mov     esi,0x0000ffff
    movd    mm3,esi
    movd    mm1,[screen_ptr]
    movzx   esi,word[xres_var]
    movd    mm2,esi
    call    lineaa
    pop     ebx
    pop     edi
 ;   pop     esi
    pop     ecx
    add     edi,12
    add     ebx,6
 ;   add     esi,12
    sub     ecx,1
    jnz     @b
;===============write triangles normals end==========
end if
    popad
    add esp,64
    pop ebp
ret

endp
gdraw_th:
  stdcall g_draw
  invoke  SendMessage,hwnd,WM_FINISH,0,0
  invoke  ExitThread,0
ret
translate_and_scale_vert:
;    ecx - count
;    esi - vert
;    xmm0 - vector as 2 dword integer ,(low qword)
;    xmm3 - scale as dword float
    punpcklwd    xmm0,[the_zero]
    cvtdq2ps     xmm0,xmm0
    shufps       xmm3,xmm3,0
  @@:
    movups   xmm1,[esi]
    mulps    xmm1,xmm3
    addps    xmm1,xmm0
    movlps   [esi],xmm1
    movhlps  xmm1,xmm1
    movss    [esi+8],xmm1
    add      esi,12
    sub      ecx,1
    jnz      @b
ret

normalize_object1:  ; procedure don't save registers !!
        .points_count equ ebp+4
        .points_r     equ ebp+8

     ;  push  ebp
       mov   ebp,esp

       xor    ebx,ebx
       xorps  xmm7,xmm7    ;xmm7 - maxxx, max of all maxes
    .find_again_min_max:
       mov    ecx,[.points_count]
       dec    ecx
       mov    edi,[.points_r]
       movss  xmm0,dword[edi+ebx]
       movss  xmm1,dword[edi+ebx]
     @@:
       maxss  xmm0,dword[edi+ebx+12]    ; xmm0 - max
       minss  xmm1,dword[edi+ebx+12]    ; xmm1 - min

       add    edi,12
       loop   @b

       addss  xmm0,xmm1             ; xmm0 - substraction
       mov    eax,2
       cvtsi2ss xmm2,eax
       divss  xmm0,xmm2             ; xmm0 - center



       mov    edi,[.points_r]
       mov    ecx,[.points_count]
    @@:
       movss  xmm2,[edi+ebx]   ; xmm0 - center
       subss  xmm2,xmm0
       movss  [edi+ebx],xmm2
       maxss  xmm7,xmm2        ; xmm7 - maximum of all
       add    edi,12           ; cooficientes
       loop   @b
       add    ebx,4
       cmp    ebx,12
       jne    .find_again_min_max

       mov    edi,[.points_r]
       mov    ecx,[.points_count]
       rcpss  xmm7,xmm7
       shufps xmm7,xmm7,0
     @@:
       movups xmm6,[edi]
       mulps  xmm6,xmm7
       movlps [edi],xmm6
       psrldq xmm6,8
       movss  [edi+8],xmm6
       add    edi,12
       loop   @b

       mov    esp,ebp
     ;  pop    ebp
ret 8
 fill_aligned:
;in  eax - dword value to fill,
;    ecx - area size in dwords
;    edi - mem adress
;    addres must be aligned
    mov  ebx,ecx
    and  ebx,111111b  ;
    jz   .fill_sse
  @@:
    mov  [edi],eax
    add  edi,4
    sub  ebx,1
    jnz  @b
  .fill_sse:
    shr    ecx,4
    or     ecx,ecx
    jz     .end
    movd   xmm0,eax
    shufps xmm0,xmm0,0
    movaps xmm1,xmm0
    movaps xmm2,xmm0
    movaps xmm3,xmm0
  @@:
    movaps  [edi],xmm0
    movaps  [edi+16],xmm1
    movaps  [edi+32],xmm2
    movaps  [edi+48],xmm3
    add     edi,64
    sub     ecx,1
    jnz     @b
  .end:
ret



rotor:
 ;  esi -  points/normals to rotate
 ;  edi -  rotated points/normals
 ;  ecx -  points(vertices)/normal vectors count
 .again_rot:
;    movlps xmm0,[esi+4]
;    shufps xmm0,xmm0,01000100b
;    movaps xmm1,[cos_gamma]
;    shufps xmm1,xmm1,00110100b
;    mulps  xmm0,xmm1
;    haddps xmm0,xmm0
;    movlps [edi+4],xmm0
    movups xmm2,[esi]
    movlps [edi],xmm2
    movhlps xmm2,xmm2
    movss  [edi+8],xmm2
  ;  push   dword[esi]
  ;  pop    dword[edi]

    movups xmm2,[edi]
    shufps xmm2,xmm2,10001000b
    movaps xmm3,[cos_beta]
    shufps xmm3,xmm3,00011100b
    mulps  xmm3,xmm2
    haddps xmm3,xmm3
;
    movss  [edi+8],xmm3
    psrldq xmm3,4
    movss  [edi],xmm3
    add    esi,12
    add    edi,12
    sub    ecx,1
    jnz    .again_rot
ret



init_triangles_normals2:
        fninit
        mov     ebx,triangles_normals
        mov     ebp,triangles
        movzx   ecx,[triangles_count_var]
     @@:
        push    ecx
        push    ebx
        mov     ebx,vectors
        movzx   esi,word[ebp]          ; first point index
        lea     esi,[esi*3]
     ; esi - pointer to 1st 3d point
        shl     esi,2
        add     esi,points_r
        movzx   edi,word[ebp+2]          ; first point index
        lea     edi,[edi*3]
        shl     edi,2
        add     edi,points_r
     ; second point index
     ; edi - pointer to 2nd 3d point

        call    make_vector_r
        add     ebx,12
        mov     esi,edi
        movzx   edi,word[ebp+4]        ; third point index
        lea     edi,[edi*3]
        shl     edi,2
        add     edi,points_r
        call    make_vector_r
        mov     edi,ebx                 ; edi - pointer to 2nd vector
        mov     esi,ebx
        sub     esi,12                  ; esi - pointer to 1st vector
        pop     ebx
        call    cross_product
        mov     edi,ebx
        call    normalize_vector
        add     ebp,6
        add     ebx,12
        pop     ecx
        loop    @b
ret

init_point_normals:
.x equ dword [ebp-4]
.y equ dword [ebp-8]
.z equ dword [ebp-12]
.point_number equ word [ebp-26]
.hit_faces    equ word [ebp-28]

        fninit
        mov       ebp,esp
        sub       esp,28
        mov       edi,points_normals ;[points_normals_ptr]
        mov       .point_number,0
    .ipn_loop:
        mov       .hit_faces,0
        mov       .x,0
        mov       .y,0
        mov       .z,0
        mov       esi,triangles;[triangles_ptr]
        xor       ecx,ecx              ; ecx - triangle number
    .ipn_check_face:
        xor       ebx,ebx              ; ebx - 'position' in one triangle
    .ipn_check_vertex:
        movzx     eax,word[esi+ebx]    ;  eax - point_number
        cmp       ax,.point_number
        jne       .ipn_next_vertex
        push      esi
        mov       esi,ecx
        lea       esi,[esi*3]
        shl       esi,2
        add       esi,triangles_normals

        fld       .x
        fadd      dword[esi]
        fstp      .x
        fld       .y
        fadd      dword[esi+4]
        fstp      .y
        fld       .z
        fadd      dword[esi+8]
        fstp      .z
        pop       esi
        inc       .hit_faces
        jmp       .ipn_next_face
    .ipn_next_vertex:
        add       ebx,2
        cmp       ebx,6
        jne       .ipn_check_vertex
    .ipn_next_face:
        add       esi,6
        inc       ecx
        cmp       cx,[triangles_count_var]
        jne       .ipn_check_face

        fld       .x
        fidiv     .hit_faces
        fstp      dword[edi]
        fld       .y
        fidiv     .hit_faces
        fstp      dword[edi+4]
        fld       .z
        fidiv     .hit_faces
        fstp      dword[edi+8]
        call      normalize_vector
        add       edi,12  ;type vector 3d
        inc       .point_number
        mov       dx,.point_number
        cmp       dx,[points_count_var]
        jne       .ipn_loop

        mov       esp,ebp
ret
norm_aligned_lv:     ; normalise aligned light vectors
;  in edi -> lights_ptr
;     ecx -> lights count
    @@:
     movaps  xmm0,[edi]
     movaps  xmm1,[edi]
     mulps   xmm1,xmm1
     haddps  xmm1,xmm1
     haddps  xmm1,xmm1
     rsqrtps xmm1,xmm1
     mulps   xmm0,xmm1
     movaps  [edi],xmm0
     add     edi,64
     sub     ecx,1
     jnz     @b
ret

random:
;  in  - ecx - min
;        edx - max
;  out - eax - random number
         mov    bx,[rand_seed]
         add    bx,0x9248
         ror    bx,3
         mov    [rand_seed],bx

         mov    ax,dx
         sub    ax,cx
         mul    bx
         mov    ax,dx
         add    ax,cx
         cwde
ret

random_rays:

   movups  xmm0,[init_rays_vert]      ; intial points
   movups  [rays_vertices],xmm0
   movups  xmm0,[init_rays_vert+12]
   movups  [rays_vertices+12],xmm0

   mov  ecx,2  ; skip two intial vert
  @@:
   push ecx
   mov  ecx,180
   mov  edx,256
   call random
   cvtsi2ss  xmm0,eax
   mov  ecx,250
   mov  edx,420
   call random
   cvtsi2ss  xmm1,eax
   mov  ecx,0
   mov  edx,53
   call random
   cvtsi2ss  xmm2,eax
   pop  ecx
   mov  ebx,ecx
   imul ebx,[i12]
   add  ebx,rays_vertices
   movss  [ebx],xmm0
   movss  [ebx+4],xmm1
   movss  [ebx+8],xmm2
   add    ecx,1
   cmp    ecx,RAYS_COUNT shr 2
   jnz    @b

  @@:
   push ecx
   mov  ecx,1
   mov  edx,85
   call random
   cvtsi2ss  xmm0,eax
   mov  ecx,246
   mov  edx,420
   call random
   cvtsi2ss  xmm1,eax
   mov  ecx,0
   mov  edx,53
   call random
   cvtsi2ss  xmm2,eax
   pop  ecx
   mov  ebx,ecx
   imul ebx,[i12]
   add  ebx,rays_vertices
   movss  [ebx],xmm0
   movss  [ebx+4],xmm1
   movss  [ebx+8],xmm2
   add    ecx,1
   cmp    ecx,RAYS_COUNT shr 1
   jnz    @b


   @@:
   push ecx
   mov  ecx,180
   mov  edx,256
   call random
   cvtsi2ss  xmm0,eax
   mov  ecx,250
   mov  edx,420
   call random
   cvtsi2ss  xmm1,eax
   mov  ecx,-70
   mov  edx,-20
   call random
   cvtsi2ss  xmm2,eax
   pop  ecx
   mov  ebx,ecx
   imul ebx,[i12]
   add  ebx,rays_vertices
   movss  [ebx],xmm0
   movss  [ebx+4],xmm1
   movss  [ebx+8],xmm2
   add    ecx,1
   cmp    ecx,(RAYS_COUNT shr 2) * 3
   jnz    @b

  @@:
   push ecx
   mov  ecx,1
   mov  edx,85
   call random
   cvtsi2ss  xmm0,eax
   mov  ecx,246
   mov  edx,420
   call random
   cvtsi2ss  xmm1,eax
   mov  ecx,-73
   mov  edx,-20
   call random
   cvtsi2ss  xmm2,eax
   pop  ecx
   mov  ebx,ecx
   imul ebx,[i12]
   add  ebx,rays_vertices
   movss  [ebx],xmm0
   movss  [ebx+4],xmm1
   movss  [ebx+8],xmm2
   add    ecx,1
   cmp    ecx,RAYS_COUNT + 2;shr 1
   jnz    @b

   mov    ebx,rays
   mov    ecx,2
 @@:
   mov    word[ebx],0
   mov    word[ebx+2],cx
   add    ebx,4
   add    ecx,1
   cmp    ecx,RAYS_COUNT shr 2
   jnz    @b

   mov    ecx,RAYS_COUNT shr 2
 @@:
   mov    word[ebx],0
   mov    word[ebx+2],cx
   add    ebx,4
   add    ecx,1
   cmp    ecx,RAYS_COUNT shr 1
   jnz    @b


 @@:
   mov    word[ebx],1
   mov    word[ebx+2],cx
   add    ebx,4
   add    ecx,1
   cmp    ecx,RAYS_COUNT
   jnz    @b

ret


include '3DMATH.INC'
include 'procs_a.inc'
include 'draw.inc'
include 'line_grd_trans.inc'
include '3bi_fil.inc'

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; data
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
section '.data' data readable writeable
    _class    TCHAR 'grtest',0
    _title    TCHAR 'Jezu Ufam Tobie, press space -> speed - full/idle ',0
    _error    TCHAR 'Startup failed.',0

    screen_size dd  ?
    rand_seed   dw  0x3e
    ccounter    dd  30
    hheap       dd  ?
    i3          dd  3
    i256        dw  256
    i255d       dd  255
    i12         dd  12
    i6          dd  6
    speed_flag  db  1
    scale_dir   db  0
    scale_cnt   dw  350
    scale_inc   dd  1.0017      ; scale increment
    scale_dec   dd  0.998302885  ; scale decrement

    scale       dd 200.0
    xxadd:      dw ?
    yyadd       dw ?
    zzadd       dw 0
                dw 0
;  piD256:
    one_deg     dd  0.01227
    aprox       dd  1.1
align 16
    align 16
    ray_clip1: dd  RAYS_COUNT shr 1
               dd  RAYS_COUNT shr 1 , 0, 0
    ray_clip2: dd  RAYS_COUNT shr 2
               dd  (RAYS_COUNT shr 2) * 3, 0, 0
 word_max_f:     times 4  dd 65535.0
 tex_m2:         dd 254.0, 423.0,  0.0, 0.0
 mask_255f:      dd 255.0, 255.0, 255.0, 255.0
 the_zero:       dd 0, 0, 0, 0
 the_one:        dd 1.0, 1.0, 1.0, 1.0
 mask_h_zero:    dd -1, -1, -1, 0
 abs_mask:
 abs_val:        dd 0x7fffffff, 0x7fffffff, 0x7fffffff, 0x7fffffff
 color1:         dd 0,0,255.0,0.05
 color2:         dd 255.0,255.0,255.0, 0.2
 color3:         dd 255.0,32.0,32.0,0.05
; ln_xadd1:       dd 1,0
; ln_yadd1:       dd 0,1

align 16
LIGHTS_COUNT = (lights_aligned_end - lights_aligned)/64
  lights_aligned:
    .light_vect    dd   0.0, 0.0, -1.0, 0.0
    .light_c_o:    dd   55.0, 49.0, 44.0, 0.0
    .light_c_min:  dd   30.0, 41.0, 30.0, 0.0
    .light_c_max:  dd   255.0, 255.0, 5.0, 0.0

      dd   -0.5,-0.5,-1.0, 0.0
      dd   32.0, 27.0, 17.0, 0.0
      dd   14.0,21.0,31.0,0.0
      dd   255.0,15.0,255.0,0.0


       dd   0.5,0.5,-1.0,0.0
       dd   13.0,1.0,80.0,0.0
       dd   7.0,33.0,17.0,0.0
       dd   25.0, 255.0, 255.0, 0.0

if 0

       dd   -0.5,0.5,-1.0,0.0
       dd   13.0,1.0,40.0,0.0
       dd   7.0,3.0,17.0,0.0
       dd   25.0, 55.0, 255.0, 0.0


       dd   0.5,-0.5,-1.0,0.0
       dd   13.0,1.0,28.0,0.0
       dd   7.0,3.0,17.0,0.0
       dd   25.0, 255.0, 55.0, 0.0
end if


  lights_aligned_end:

points_count_var         dw 8
triangles_count_var      dw 12
rays_count_var           dw RAYS_COUNT
points_normals_count_var dw  8


init_rays_vert:
dd 137.0, 110.0, 0.0
dd 118.0, 110.0, -20.0
points_tex:
dw 0,0, 255,0,0,424, 255,424
dw 255,0,0,0,255,424, 0,424

triangles:
;tetra_triangles:
dw 0,1,2, 1,3,2
dw 4,6,5, 5,6,7
dw 1,0,4, 4,5,1
dw 3,1,5, 5,7,3
dw 2,3,6, 6,3,7
dw 0,2,6, 4,0,6

jut:
   file 'jut256x425.raw'

    msg  MSG
    rect RECT
    wc WNDCLASS 0,WindowProc,0,0,NULL,NULL,NULL,1,NULL,_class
points_r:
;tetra_points_r:
dd 0.0, 0.0, 0.0
dd 256.0, 0.0, 0.0
dd 0.0, 425.0,  0.0
dd 256.0, 425.0, 0.0

dd 0.0, 0.0, -20.0
dd 256.0, 0.0, -20.0
dd 0.0, 425.0,  -20.0
dd 256.0, 425.0, -20.0
rays_vertices:
  rd 12*(RAYS_COUNT + 2)

rays:
  rd RAYS_COUNT


rays_vertices_rotated:
  rd 12*(RAYS_COUNT + 2)

    screen_ptr       dd ?
    counter_buff_ptr dd ?
    ptrs_history_ptr dd ?
    Zbuffer_ptr      dd ?
    xres_var:        dw ?
    yres_var         dw ?
    history_ptr      dd ?

align 16
    cos_beta:     rd 1
    sin_beta      dd ?
    neg_cos_beta  dd ?
    neg_sin_beta  dd ?

    cos_gamma:    rd 1
    sin_gamma     dd ?
    neg_cos_gamma dd ?
    neg_sin_gamma dd ?

    hwnd      dd ?
    hdc       dd ?
    bmi       BITMAPINFOHEADER

    texmap    rd TEX_X * TEX_Y
    points_rotated:
       rd 120
    triangles_normals:
       rd 120
    triangles_normals_rotated:
       rd 120
    points_normals:
       rd 120
    points_normals_rotated:
       rd 120

    vectors     rb 24
    ThreadID0   dd ?
    ThreadID1   dd ?
    ThreadID2   dd ?
    ThreadID_gdraw dd ?
    hthread0       rd 1
    hthread1       rd 1
    hthread_gdraw  rd 1

section '.idata' import data readable writeable
    library kernel32,'KERNEL32',\user32,'USER32',\gdi32,'GDI32',\winmm,'WINMM',\
    comdlg32,'COMDLG32'    ;   timing,'timing.dll'
    include '\include\api\kernel32.inc'
    include '\include\api\user32.inc'
    include '\include\api\gdi32.inc'
    include '\include\api\comdlg32.inc'
    import winmm,timeGetTime,'timeGetTime'